Package: prettycode
Title: Pretty Print R Code in the Terminal
Version: 1.1.0
Author: Gábor Csárdi
Maintainer: Gábor Csárdi <csardi.gabor@gmail.com>
Description: Replace the standard print method for functions with one that
    performs syntax highlighting, using ANSI colors, if the terminal
    supports them.
License: MIT + file LICENSE
LazyData: true
URL: https://github.com/r-lib/prettycode#readme
BugReports: https://github.com/r-lib/prettycode/issues
RoxygenNote: 6.1.0
Imports: crayon, utils
Suggests: covr, mockery, rstudioapi, testthat, withr
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2019-12-16 13:01:04 UTC; gaborcsardi
Repository: CRAN
Date/Publication: 2019-12-16 14:00:06 UTC
